/*
 * Copyright 1990,91 by Thomas Roell, Dinkelscherben, Germany.
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Thomas Roell not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Thomas Roell makes no representations
 * about the suitability of this software for any purpose.  It is provided
 * "as is" without express or implied warranty.
 *
 * THOMAS ROELL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL THOMAS ROELL BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 *
 * Author:  Thomas Roell, roell@informatik.tu-muenchen.de
 *
 * $Header: /proj/X11/mit/server/ddx/at386/vga/RCS/vga.h,v 1.6 91/02/10 16:44:35 root Exp $
 */

#define SINGLEWINDOW

extern void    vgaConfig();
extern Bool    vgaScreenInit();
extern void    vgaEnterVT();
extern void    vgaLeaveVT();
extern void    vgaAdjustFrame();
extern Bool    vgaSwitchMode();

extern void    vgaSaveScreen();
extern void    vgaCloseScreen();
extern void    vgaStoreColor();
extern void    vgaBitBlt();
extern void    vgaImageRead();
extern void    vgaImageWrite();
extern void    vgaPixBitBlt();
extern void    vgaImageGlyphBlt();

extern void    vgaHWInit();
extern void    vgaHWRestore();
extern void *  vgaHWSave();

/*
 * structure for accessing the video chip`s functions
 */
typedef struct {
  void (* ChipInit)();
  void * (* ChipSave)();
  void (* ChipRestore)();
  void (* ChipAdjust)();
  void (* ChipSetRead)();
  void (* ChipSetWrite)();
  void (* ChipSetReadWrite)();
  int ChipMapSize;
  int ChipSegmentSize;
  int ChipSegmentShift;
  int ChipSegmentMask;
  int ChipReadBottom;
  int ChipReadTop;
  int ChipWriteBottom;
  int ChipWriteTop;
  
} vgaVideoChipRec, *vgaVideoChipPtr;

extern vgaVideoChipRec WDI;
extern vgaVideoChipRec ET3000;
extern vgaVideoChipRec ET4000;
extern vgaVideoChipRec GVGA;
extern vgaVideoChipRec TVGA8900;
extern vgaVideoChipRec ATI;
extern vgaVideoChipRec C452;
extern vgaVideoChipRec C453;

/*
 * hooks for communicating with the VideoChip on the VGA
 */
extern void (* vgaInitFunc)();
extern void * (* vgaSaveFunc)();
extern void (* vgaRestoreFunc)();
extern void (* vgaAdjustFunc)();
extern void (* vgaSetReadFunc)();
extern void (* vgaSetWriteFunc)();
extern void (* vgaSetReadWriteFunc)();
extern int vgaMapSize;
extern int vgaSegmentSize;
extern int vgaSegmentShift;
extern int vgaSegmentMask;
extern void *vgaReadBottom;
extern void *vgaReadTop;
extern void *vgaWriteBottom;
extern void *vgaWriteTop;
extern int vgaIOBase;

/*
 * These routines implement the whole VGA banking
 */
extern void * vgaSetReadWrite();
extern void * vgaReadWriteNext();
extern void * vgaReadWritePrev();
extern void * vgaSetRead();
extern void * vgaReadNext();
extern void * vgaReadPrev();
extern void * vgaSetWrite();
extern void * vgaWriteNext();
extern void * vgaWritePrev();
extern void vgaPushRead();
extern void vgaPopRead();

#define VGABASE 0xF0000000

/*
 * structure common for all modes
 */
typedef struct {
  char *         name;               /* identifier of this mode */
  int            Clock;              /* doclock */
  int            HDisplay;           /* horizontal timing */
  int            HSyncStart;
  int            HSyncEnd;
  int            HTotal;
  int            VDisplay;           /* vertical timing */
  int            VSyncStart;
  int            VSyncEnd;
  int            VTotal;
  int            Flags;
} vgaModeRec, *vgaModePtr;

#define V_FASTRAM   0x0001
#define V_PHSYNC    0x0002
#define V_NHSYNC    0x0004
#define V_PVSYNC    0x0008
#define V_NVSYNC    0x0010
#define V_INTERLACE 0x0020

#define C_LEGEND    0x0001
#define C_SYNCCLUT  0x0002

#define NOCLOCKS 32
#define NOMODES  16

extern int vgaVideoRam;
extern int vgaVideoClock[NOCLOCKS];         /* quarz oszialtors on VGA board */
extern int vgaVideoClocks;
extern vgaModeRec vgaVideoModes[NOMODES]; /* allow more modes */
extern int vgaNoVideoModes;
extern int vgaCurrentVideoMode;
extern int vgaChipSetFlags;       /* keep up with special implementations */
extern pointer vgaOrigVideoState;    /* buffers for all video information */
extern pointer vgaNewVideoState;
extern pointer vgaBase;              /* the framebuffer himself */


typedef struct {
  unsigned char MiscOutReg;     /* */
  unsigned char CRTC[25];       /* Crtc Controller */
  unsigned char Sequencer[5];   /* Video Sequencer */
  unsigned char Graphics[9];    /* Video Graphics */
  unsigned char Attribute[21];  /* Video Atribute */
  unsigned char DAC[768];       /* Internal Colorlookuptable */
  unsigned char NoClock;        /* number of selected clock */
  pointer FontInfo;             /* save area for fonts */ 
} vgaHWRec, *vgaHWPtr;


#ifndef SINGLEWINDOW
#define SETR(f,x)  { if(f) x = vgaSetRead(x); }
#define SETW(f,x)  { if(f) x = vgaSetWrite(x); }
#else
#define SETR		SETRW
#define SETW		SETRW
#endif

#define SETRW(f,x) { if(f) x = vgaSetReadWrite(x); }

#ifndef SINGLEWINDOW
#define CHECKRO(f,x) { if(f && ((void *)x >= vgaReadTop)) \
			 x = vgaReadNext(x); }
#define CHECKRU(f,x) { if(f && ((void *)x < vgaReadBottom)) \
			 x = vgaReadPrev(x); }
#define CHECKWO(f,x) { if(f && ((void *)x >= vgaWriteTop)) \
			 x = vgaWriteNext(x); }
#define CHECKWU(f,x) { if(f && ((void *)x < vgaWriteBottom)) \
			 x = vgaWritePrev(x); }
#else
#define CHECKRO		CHECKRWO
#define CHECKRU		CHECKRWU
#define CHECKWO		CHECKRWO
#define CHECKWU		CHECKRWU

#endif

#define CHECKRWO(f,x) { if(f && ((void *)x >= vgaWriteTop)) \
			  x = vgaReadWriteNext(x); }
#define CHECKRWU(f,x) { if(f && ((void *)x < vgaWriteBottom)) \
			  x = vgaReadWritePrev(x); }

#ifndef SINGLEWINDOW
#define PUSHR(f)     { if(f) vgaPushRead(); }
#define POPR(f)      { if(f) vgaPopRead(); }

#define NEXTR(x) { x = vgaReadNext(x);}
#define NEXTW(x) { x = vgaWriteNext(x); }
#define PREVR(x) { x = vgaReadPrev(x); }
#define PREVW(x) { x = vgaWritePrev(x); }
#else
#define PUSHR(f)
#define POPR(f)

#define NEXTR(x) { x = vgaReadWriteNext(x);}
#define NEXTW(x) { x = vgaReadWriteNext(x); }
#define PREVR(x) { x = vgaReadWritePrev(x); }
#define PREVW(x) { x = vgaReadWritePrev(x); }

#endif

#define CHECKSCREEN(x) (((unsigned long)x >= VGABASE) ? TRUE : FALSE)

#define BITS_PER_GUN 6
#define COLORMAP_SIZE 256

extern void vgaImageGlyphBlt();
extern void vgaDoBitBlt();
